; File Name:         BUGGED.ASM
; Revision:                1.03
; Date:      20th January, 1987
; Author:          G.W. Haywood

; Monitor for the M6800 like MIKBUG, modified somewhat by Ged.
; Taken from the 4th January 1978 paper tape version....
; This version is set up for the HYDRAGAMMA 16 and its descendants.
;
; -----------------------------------------------------------------------------
; Modification record:
; 15.01.87  Added extra commands A B C S X.
; 19.01.87  Corrected ACIA initialisation: should be Clock/1 not /16.
;           Added a useful message at the bottom.  Removed all TAB characters.
; 20.01.87  Corrected a couple of bugs.
; -----------------------------------------------------------------------------
;
;                    6800 STACK BEFORE & AFTER AN INTERRUPT
;                    ======================================
;
;     |               |
;     |  High memory  |
;     |_______________|
;     |               |
;     |   PC - LOW    |   <====  STACK POINTER POINTS HERE BEFORE THE INTERRUPT
;     |_______________|
;     |               |
;     |   PC - HIGH   |
;     |_______________|
;     |               |
;     | INDEX - LOW   |
;     |_______________|
;     |               |    It is very important to understand this diagram
;     | INDEX - HIGH  |    because the stack organisation is relied upon in
;     |_______________|    order to execute the change register and Go commands.
;     |               |
;     | ACCUMULATOR A |    N.B. of course the register contents are not
;     |_______________|    contained in the RAM locations shown here until
;     |               |    after the interrupt has occurred.  Before then
;     | ACCUMULATOR B |    the contents could be anything.
;     |_______________|
;     |               |
;     |  CONDITIONS   |
;     |_______________|
;     |               |
;     |  NEXT EMPTY   |   <====  STACK POINTER POINTS HERE AFTER THE INTERRUPT
;     |_______________|
;     |               |
;     |  Low memory   |
;     |               |
;
; -----------------------------------------------------------------------------
;
;   LIST OF COMMANDS AVAILABLE:
;
; A NN        EXAMINE/CHANGE ACCUMULATOR A ON TARGET STACK
; B NN        EXAMINE/CHANGE ACCUMULATOR B ON TARGET STACK
; C NN        EXAMINE/CHANGE CONDITION CODES ON TARGET STACK
; D NNNN DDDD MEMORY DUMP
; E NNNN      EXAMINE/CHANGE MEMORY
; G NNNN      GO TO TARGET PROGRAM
; L           LOAD  (S9 WILL GET YOU BACK IF YOU HIT IT BY ACCIDENT)
; R           DISPLAY CONTENTS OF TARGET STACK
; S NNNN      EXAMINE/CHANGE STACK POINTER ON TARGET STACK
; X NNNN      EXAMINE/CHANGE INDEX REGISTER ON TARGET STACK

NAME   "BUGGED.ASM"

;       **************************
;       * RAM usage definitions. *
;       **************************

; There's 1K of static RAM at 800-BFF and 256 bytes at 0000-00FF.
; The two 128 byte chips at 0000 and 0080 suffer DMA transfers
; when the instrument is counting, so we don't use them here.
        ORG      $800
IOV     DW      0       ;IRQ interrupt pointer
NIOV    DW      0       ;NMI interrupt pointer
SFTV    DW      0       ;SWI interrupt pointer
SP      DW      0       ;Stack pointer for target
BEGA    DW      0       ;begining addr print/punch
ENDA    DW      0       ;ending addr. print/punch
CKSM    DB      0       ;checksum
BYTECT  DB      0       ;byte count
XHI     DB      0       ;xreg high
XLOW    DB      0       ;xreg low
TEMP    DB      0       ;char count (inadd)
TW      DW      0       ;temp
MCONT   DB      0       ;temp
XTEMP   DW      0       ;x-reg temp storage
XTEMP2  DW      0       ;ditto

        ORG     $0BEF
STACK   DB      0       ;stack pointer for this program

;       *********************************
;       * End of RAM usage definitions. *
;       *********************************

;       *********************************
;       * I/O port for comms with user. *
;       *********************************

ACIASR  EQU     $8014   ;SERIAL INTERFACE STATUS REGISTER
ACIACR  EQU     $8014   ;SERIAL INTERFACE CONTROL REGISTER
ACIADR  EQU     $8015   ;SERIAL INTERFACE DATA REGISTERS

;      *************************************
;      * The program code starts here, but *
;      * the entry point is label START.   *
;      *************************************

        ORG $F000
; I/O (IRQ) INTERRUPT SEQUENCE
IO      LDX     IOV
        JMP     0,X

; NMI SEQUENCE
POWDWN  LDX     NIOV    ;GET NMI VECTOR
        JMP     0,X


LOAD    EQU     $
; 15.01.87 commented out 2 lines
;        LDAA   #$51
;        STAA   ACIACR  ;READER RELAY ON
        LDAA    #$11
        BSR     OUTCH  ;OUTPUT CHAR


LOAD3   BSR     INCH
        CMPA    #"S"
        BNE     LOAD3   ;1ST CHAR NOT (S)
        BSR     INCH    ;READ CHAR
        CMPA    #"9"
        BEQ     LOAD21
        CMPA    #"1"
        BNE     LOAD3   ;2ND CHAR NOT (1)
        CLR     CKSM    ;ZERO CHECKSUM
        BSR     BYTE    ;READ BYTE
        SUBA    #2
        STAA    BYTECT  ;BYTE COUNT
; BUILD ADDRESS
        BSR     BADDR
; STORE DATA
LOAD11  BSR     BYTE
        DEC     BYTECT
        BEQ     LOAD15  ;ZERO BYTE COUNT
        STAA    0,X     ;STORE DATA
        INX
        BRA     LOAD11


LOAD15  INC     CKSM
        BEQ     LOAD3
LOAD19  LDAA    #"?"    ;PRINT QUESTION MARK
        BSR     OUTCH
LOAD21  EQU     $
C1      JMP     CONTRL


;       BUILD ADDRESS
BADDR   BSR     BYTE    ;READ TWO FRAMES
        STAA    XHI
        BSR     BYTE
        STAA    XLOW
        LDX     XHI     ;(X) ADDRESS WE BUILT
        RTS


;INPUT  BYTE (TWO FRAMES)
BYTE    BSR     INHEX   ;GET HEX CHAR
BYTE1   ASLA
        ASLA
        ASLA
        ASLA
        TAB
        BSR     INHEX
        ABA
        TAB
        ADDB    CKSM
        STAB    CKSM
        RTS


OUTHL   LSRA            ;OUT HEX LEFT BCD DIGIT
        LSRA
        LSRA
        LSRA
        
        
OUTHR   ANDA    #$F     ;OUT HEX RIGHT BCD DIGIT
        ADDA    #$30
        CMPA    #$39
        BLS     OUTCH
        ADDA    #$7


;     OUTPUT ONE CHAR
OUTCH  JMP    OUTEEE
INCH   JMP    INEEE


;PRINT DATA POINTED AT BY X-REG
PDATA2 BSR    OUTCH
       INX
PDATA1 LDAA   0,X
       CMPA   #4
       BNE    PDATA2
       RTS              ;STOP ON EOT


;CHANGE MEMORY (M AAAA DD NN)
CHANGE BSR    BADDR     ;BUILD ADDRESS
CHA51  LDX    #MCL
       BSR    PDATA1    ;C/R L/F
       LDX    #XHI
       BSR    OUT4HS    ;PRINT ADDRESS
       LDX    XHI
       BSR    OUT2HS    ;PRINT DATA (OLD)
       STX    XHI       ;SAVE DATA ADDRESS
       BSR    INCH      ;INPUT ONE CHAR
       CMPA   #$20
;15.01.87  Two lines replaced.
;       BNE    CHA51
       BEQ    CHA51     ;Space now causes skip to next location
;       BSR    BYTE      ;INPUT NEW DATA
       BSR    INHX1
       BSR    BYTE1
       DEX
       STAA   0,X       ;CHANGE MEMORY
       CMPA   0,X
       BEQ    CHA51     ;DID CHANGE
       BRA    LOAD19    ;NOT CHANGED


;INPUT HEX CHAR
INHEX   BSR    INCH
INHX1   SUBA   #$30
        BMI    NOTHEX   ;NOT HEX
        CMPA   #$09
        BLE    IN1HG
        CMPA   #$11
        BMI    NOTHEX
        CMPA   #$16
        BGT    NOTHEX
        SUBA   #7
IN1HG   RTS

NOTHEX  LDAA   #7       ;Ring my bell
        JSR    OUTEEE
        JMP    CONTRL

OUT2H   LDAA  0,X       ;OUTPUT 2 HEX CHAR
OUT2HA  BSR    OUTHL    ;OUT LEFT HEX CHAR
        LDAA  0,X
        INX
        BRA    OUTHR    ;OUTPUT RIGHT HEX CHAR AND RETURN


OUT4HS  BSR    OUT2H    ;OUTPUT 4 HEX CHAR + SPACE
OUT2HS  BSR    OUT2H    ;OUTPUT 2 HEX CHAR + SPACE
OUTS    LDAA   #$20     ;SPACE
        BRA    OUTCH    ;(BSR & RTS)


;          ****************************
;          * PROGRAM COLD ENTRY POINT *
;          ****************************

START   EQU     $
        LDS     #STACK
        STS     SP      ;INZ TARGET'S STACK PNTR
        LDAA    #$FF    ;MASTER RESET ACIA
        STAA    ACIACR
;19.01.87 ACIA CLOCK/1 not /16
        LDAA    #$14    ;SET FOR TTY OPERATION,NO INTERRUPTS, CLOCK/1
        STAA    ACIACR  ;READER OFF
        LDX     #POWDWN ;The vectors are all aranged as jumps through RAM
        STX     NIOV    ;so that this vector can be changed by the user to
        LDX     #SFE    ;point to his own routines.
        STX     SFTV
        LDX     #IO
        STX     IOV
        LDX     #INTRO  ;Print helpful message
        JSR     PDATA1
CONTRL  LDS     #STACK  ;SET CONTR STACK POINTER
        LDX     #PROMPT
        JSR     PDATA1  ;PRINT DATA STRING

        JSR     INCH    ;READ CHARACTER
        TAB
        BSR     OUTS    ;PRINT SPACE
        CMPB    #"L"
        BNE     $+5
        JMP     LOAD
        CMPB    #"E"
        BNE     $+5
        JMP     CHANGE
        CMPB    #"R"
        BNE     $+5
        JMP     PRINT   ;Dump Registers
        CMPB    #"D"
        BNE     $+5
        JMP     PUNCH   ;Display memory in a range
        CMPB    #"A"
        BEQ     REGA    ;Modify accumulator A
        CMPB    #"B"
        BEQ     REGB    ;Acc. B
        CMPB    #"C"
        BEQ     REGC    ;Condition codes
        CMPB    #"S"
        BNE     $+5
        JMP     REGS    ;Stack pointer
        CMPB    #"X"
        BNE     $+5
        JMP     REGX    ;Index reg
        CMPB    #"?"
        BNE     $+5
        JMP     HELPMS  ;Helpful message
        CMPB    #"G"
        BNE     CONTRL
        JSR     BADDR   ;Get the four-digit Go address.
        STX     XTEMP
        LDAA    XTEMP
        LDAB    XTEMP+1
        LDX     SP
        STAA    6,X
        STAB    7,X
        LDS     SP      ;restore pgm's stack ptr
        RTI             ;Go


;Modify Acc. A
REGA    LDX     SP      ;Top of target stack
        INX
        INX
        INX
        STX     XTEMP
        JSR     OUT2HS
        JSR     INHEX
        JSR     BYTE1
        LDX     XTEMP
        STAA    0,X
        JMP     CONTRL

;Modify Acc. B
REGB    LDX     SP
        INX
        INX
        STX     XTEMP
        JSR     OUT2HS
        JSR     INHEX
        JSR     BYTE1
        LDX     XTEMP
        STAA    0,X
        JMP     CONTRL

;Modify Condition codes
REGC    LDX     SP
        INX
        STX     XTEMP
        JSR     OUT2HS
        JSR     INHEX
        JSR     BYTE1
        LDX     XTEMP
        STAA    0,X
        JMP     CONTRL

;Modify indeX register
REGX    LDX     SP
        INX
        INX
        INX
        INX
        STX     XTEMP
        JSR     OUT4HS
        JSR     BADDR
        STX     XTEMP2
        LDAA    XTEMP2
        LDAB    XTEMP2+1
        LDX     XTEMP
        STAA    0,X
        STAB    1,X
        JMP     CONTRL

; Modify Stack pointer - This will effectively change all the others.
REGS    LDX     #SP
        JSR     OUT4HS
        JSR     BADDR
        STX     SP
        JMP     CONTRL

;Print helpful message.
HELPMS  LDX     #INTRO
        JSR     PDATA1
        JMP     CONTRL

;Enter from software interrupt
SFE     LDX     SFTV    ;May still point to SFE1...... or not.
        JMP     0,X
SFE1    EQU     $
        STS     SP      ;save target's stack pointer
;Decrement program counter
        TSX
        TST     6,X
        BNE     $+4
        DEC     5,X
        DEC     6,X


;PRINT CONTENTS OF STACK
PRINT   LDX     #REGHED ;print heading
        JSR     PDATA1
        LDX     SP
        INX
        JSR     OUT2HS  ;condiion codes
        JSR     OUT2HS  ;acc-B
        JSR     OUT2HS  ;acc-B
        JSR     OUT4HS  ;X-reg
        JSR     OUT4HS  ;Program counter
        LDX     #SP
        JSR     OUT4HS  ;Stack pointer
C2      JMP     CONTRL
; Heading for register printout.
REGHED  DB      0DH,0AH,"C  B  A  X    P    S"

;PUNCH DUMP
;PUNCH FROM BEGINING ADDRESS (BEGA) TO END
;ADDRESS (ENDA) ..or nearly.. GWH

MTAPE1  DB      $D,$A,4  ;PUNCH FORMAT
ENDTAP  DB      "S","9",4

PUNCH   EQU     $

        LDAA    #$12    ;TURN TTY PUNCH ON
        JSR     OUTCH   ;OUT CHAR
        JSR     BADDR   ;GET BEGINNING AND END ADDRESSES
        STX     BEGA
        JSR     OUTS
        JSR     BADDR
        STX     ENDA
        LDAA    BEGA+1  ;MUST HAVE BEGA AND ENDA MULTIPLE OF 16.
        ANDA    #$F0    ;RATHER A BRUTAL WAY OF DOING IT
        STAA    BEGA+1
        LDAA    ENDA+1
        ADDA    #$10
        ANDA    #$F0
        STAA    ENDA+1
        BCC     NOCARR
        INC     ENDA
NOCARR  LDX     BEGA
        STX     TW      ;TEMP BEGINING ADDRESS FOR DUMP
PUN11   LDAA    ENDA+1
        SUBA    TW+1
        LDAB    ENDA
        SBCB    TW
        BNE     PUN22
        CMPA    #16
        BCS     PUN23
PUN22   LDAA    #15
PUN23   ADDA    #4
        STAA    MCONT     ;FRAME COUNT THIS RECORD
        SUBA    #3
        STAA    TEMP      ;BYTE COUNT THIS RECORD
; PUNCH C/R,L/F,NULL,S,1
        LDX     #MTAPE1
        JSR     PDATA1
        CLRB              ;ZERO CHECKSUM
;15.01.87: DON'T PUNCH FRAME COUNT
;PUNCH FRAME COUNT
;        LDX     #MCONT
;        BSR     PUNT2     ;PUNCH 2 HEX CHAR
;PUNCH ADDRESS
        LDX     #TW
;15.01.87: 4 DIGITS AND A SPACE
        JSR     OUT4HS
;        BSR     PUNT2
;        BSR     PUNT2
;        PUNCH   DATA
        LDX     TW
PUN32   JSR     OUT2HS ;15.01.87: WAS PUNT2 - PUNCH ONE BYTE (2 FRAMES)
        DEC     TEMP     ;DEC BYTE COUNT
        BNE     PUN32
        STX     TW
;        COMB
;        PSHB           15.01.87: DON'T BOTHER WITH CHECKSUM
;        TSX
;        BSR     PUNT2   ;PUNCH CHECKSUM
;        PULB            ;RESTORE STACK
;        LDX     TW
;        DEX
        CPX     ENDA
        BNE     PUN11
;        LDX     #ENDTAP
;        JSR     PDATA1
        JMP     C2      ;JMP TO CONTRL


;PUNCH 2 HEX CHAR, UPDATE CHECKSUM
;PUNT2   ADDB    0,X    ;UPDATE CHECKSUM
;        JMP     OUT2H  ;OUTPUT TWO HEX CHAR AND RTS


PROMPT  DB      $13                  ;reader off, etc.
MCL     DB      $D,$A,"bugged> ",4   ;CR, LF, Prompt string

INEEE   LDAA    ACIASR  ;READ STATUS
        RORA            ;ROTATE BIT TO CARRY
        BCC     INEEE   ;BRANCH IF REGISTER NOT READY
        LDAA    ACIADR  ;LOAD CHARACTER INTO A-REG
        ANDA    #$7F    ;RESET PARITY BIT
        BSR     OUTEEE  ;ECHO CHARACTER
        RTS             ; & RETURN

OUTEEE  PSHA            ;SAVE CHARACTER
OUT1    LDAA    ACIASR  ;LOAD STATUS
        RORA            ;ROTATE BIT TO CARRY
        RORA
        BCC     OUT1    ;BRANCH IF REGISTER NOT READY
        PULA            ;RESTORE CHARACTER
        STAA    ACIADR  ;SEND CHARACTER
        RTS             ;& RETURN.

; Useful message......

INTRO   DB      $D,$A,$A
        DB      "     BUGGED   - Mickey Mouse Quality Debugging tool.",$D,$A,$A
        DB      "  A           - Examine/change accumulator A on target stack",$D,$A
        DB      "  B           - Examine/change accumulator B",$D,$A
        DB      "  C           - Examine/change Condition Codes",$D,$A
        DB      "  D bbbb eeee - Display a range of memory locations",$D,$A
        DB      "  E mmmm      - Examine/change memory location from mmmm",$D,$A
        DB      "  G nnnn      - Go from address nnnn.  SET UP CC FIRST!",$D,$A
        DB      "  L           - Load memory (from paper tape... S9 to quit)",$D,$A
        DB      "  R           - Display all Registers",$D,$A
        DB      "  S           - Examine/change Stack pointer",$D,$A
        DB      "  X           - Examine/change indeX register",$D,$A
        DB      "  ?           - Print this message",$D,$A,4

;Startup and interrupt vectors.
        ORG     $FFF8
        DW      IO
        DW      SFE
        DW      POWDWN
        DW      START

        END
